"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const v8 = require("v8");
const utils_1 = require("./utils");
const logger_1 = require("./utils/logger");
const registry_1 = require("./ddf-rules/registry");
const ddf_json_corrector_1 = require("./ddf-definitions/ddf-json-corrector");
const index_1 = require("./index");
const version_1 = require("./version");
const localPackage = require('./package.json');
let isValidationExpected = true;
if (utils_1.settings.versionShouldBePrinted) {
    console.log(localPackage.version);
    isValidationExpected = false;
}
if (utils_1.settings.heap) {
    v8.setFlagsFromString(`--max-old-space-size=${utils_1.settings.heap}`);
}
if (utils_1.settings.isDataPackageActual && !utils_1.settings.versionShouldBePrinted) {
    isValidationExpected = false;
    index_1.getDataPackageActuality({ ddfRootFolder: utils_1.ddfRootFolder }, (message) => {
        console.log(message);
        process.exit(0);
    });
}
if (utils_1.settings.isDataPackageGenerationMode && !utils_1.settings.versionShouldBePrinted) {
    index_1.createDataPackage({ ddfRootFolder: utils_1.ddfRootFolder }, (message) => {
        console.log(message);
    }, (err) => {
        if (err) {
            console.log(err);
            process.exit(1);
        }
    });
    isValidationExpected = false;
}
if (utils_1.settings.isJsonAutoCorrectionMode && !utils_1.settings.versionShouldBePrinted) {
    const ddfJsonCorrector = new ddf_json_corrector_1.DdfJsonCorrector(utils_1.ddfRootFolder);
    ddfJsonCorrector.correct((correctorError, csvFileDescriptors) => {
        if (correctorError) {
            utils_1.logger.notice(correctorError);
            return;
        }
        ddfJsonCorrector.write(csvFileDescriptors, (writeError) => {
            utils_1.logger.notice(writeError ? writeError : 'ok...');
        });
    });
    isValidationExpected = false;
}
if (utils_1.settings.isPrintRules && !utils_1.settings.versionShouldBePrinted) {
    utils_1.logger.notice(registry_1.getRulesInformation());
    isValidationExpected = false;
}
if (!isValidationExpected) {
    version_1.checkLatestVersion(localPackage.version);
}
if (isValidationExpected) {
    const ExpectedValidator = utils_1.settings.isSummaryNeeded || utils_1.settings.silent ? index_1.JSONValidator : index_1.StreamValidator;
    const validator = new ExpectedValidator(utils_1.ddfRootFolder, utils_1.settings);
    const issues = [];
    let hasIssue = false;
    validator.on('issue', (issue) => {
        hasIssue = true;
        if (validator instanceof index_1.StreamValidator) {
            utils_1.logger.notice(`${JSON.stringify(issue, null, 2)},\n`);
        }
        if (validator instanceof index_1.JSONValidator) {
            issues.push(issue);
        }
    });
    validator.on('finish', (err) => {
        if (err) {
            throw err;
        }
        if (!utils_1.settings.silent && !utils_1.settings.isSummaryNeeded && hasIssue) {
            console.log(`\nValidation was finished with issues. Details are here: ${logger_1.validationTransport.file}.`);
        }
        if (utils_1.settings.silent && hasIssue) {
            console.log(JSON.stringify(issues, null, 2));
        }
        if (validator instanceof index_1.JSONValidator && utils_1.settings.isSummaryNeeded && hasIssue) {
            console.log(`\n\nSummary:\n\n${JSON.stringify(validator.summary, null, 2)}`);
        }
        if (!utils_1.settings.silent && !hasIssue) {
            console.log('\nValidation was finished successfully.');
        }
        version_1.checkLatestVersion(localPackage.version, hasIssue ? 1 : 0);
    });
    index_1.validate(validator);
}
