"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CsvParser = require("papaparse");
const lodash_1 = require("lodash");
const fs_1 = require("fs");
const utils_1 = require("../utils");
const getErrors = parsedCsv => parsedCsv.errors
    .filter(error => error.row >= 0)
    .map(error => ({
    message: error.message,
    row: error.row,
    type: `${error.type}/${error.code}`,
    data: parsedCsv.data[error.row]
}));
class CsvChecker {
    constructor(filePath) {
        this.errors = [];
        this.filePath = filePath;
        this.error = null;
        this.errors = [];
    }
    check(onChecked) {
        fs_1.readFile(this.filePath, 'utf-8', (err, fileContent) => {
            if (err) {
                utils_1.logger.error(err);
                return onChecked();
            }
            CsvParser.parse(fileContent, {
                header: true,
                delimiter: ',',
                skipEmptyLines: true,
                complete: parsedCsv => {
                    this.errors = getErrors(parsedCsv);
                    return onChecked();
                }
            });
        });
    }
    isCorrect() {
        return lodash_1.isEmpty(this.errors) && !this.error;
    }
}
exports.CsvChecker = CsvChecker;
