"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const async_1 = require("async");
const Mingo = require('mingo');
const COLLECTION_NAME = 'concepts';
const PARALLEL_PROCESSES = 5;
class Concept {
    constructor(db) {
        this.db = db;
        this.collection = this.db.getCollection(COLLECTION_NAME);
        this.fileDescriptors = [];
    }
    getAllData() {
        return this.collection;
    }
    getDataIdsByType(type) {
        const query = new Mingo.Query({ concept_type: type });
        return query.find(this.collection).all().map(record => record.concept);
    }
    getRecordByKey(concept) {
        const query = new Mingo.Query({ concept });
        return query.find(this.collection).first();
    }
    getDataByFiles() {
        return lodash_1.groupBy(this.getAllData(), record => record['$$source']);
    }
    getDictionary(concepts, field) {
        const result = {};
        const query = new Mingo.Query(concepts ? { concept: { $in: concepts } } : {});
        query.find(this.collection).all().forEach(concept => {
            result[concept.concept] = concept[field];
        });
        return result;
    }
    addFileDescriptor(fileDescriptor) {
        this.fileDescriptors.push(fileDescriptor);
    }
    getIds() {
        return this.getAllData().map(record => record.concept);
    }
    getTranslationsData(onTranslationsReady) {
        const transActions = this.fileDescriptors.map(fileDescriptor => fileDescriptor.getExistingTranslationDescriptors()
            .map(translationDesc => onTranslationLoaded => {
            const transCollectionName = `translation-${translationDesc.file}`;
            this.db.fillCollection(transCollectionName, translationDesc.fullPath, err => {
                translationDesc.content = this.db.getCollection(transCollectionName);
                onTranslationLoaded(err);
            }, true);
        }));
        const transActionsNormalized = lodash_1.compact(lodash_1.flattenDeep(transActions));
        async_1.parallelLimit(transActionsNormalized, PARALLEL_PROCESSES, onTranslationsReady);
    }
}
exports.Concept = Concept;
