"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const async_1 = require("async");
const ddf_data_set_1 = require("./ddf-data-set");
const constants_1 = require("../ddf-definitions/constants");
const ddf_rules_1 = require("../ddf-rules");
const registry_1 = require("../ddf-rules/registry");
const file_1 = require("../utils/file");
const jsonExport = require('jsonexport');
const PROCESS_LIMIT = 5;
function correctFile(data, cb) {
    file_1.readFile(data.file, (err, content) => {
        if (err) {
            return cb(err);
        }
        data.result
            .filter(issue => issue.path === data.file)
            .forEach(issue => {
            const suggestion = lodash_1.first(issue.suggestions);
            if (suggestion) {
                content[issue.data.line - constants_1.LINE_NUM_INCLUDING_HEADER][issue.data.column] =
                    lodash_1.first(issue.suggestions);
            }
        });
        return cb(null, content);
    });
}
function jsonToCsv(data, onJsonReady) {
    jsonExport(data.content, onJsonReady);
}
class DdfJsonCorrector {
    constructor(folder) {
        this.folder = folder;
    }
    correct(onJsonCorrected) {
        const ddfDataSet = new ddf_data_set_1.DdfDataSet(this.folder, null);
        ddfDataSet.load(() => {
            const result = ddf_rules_1.allRules[registry_1.INCORRECT_JSON_FIELD].rule(ddfDataSet);
            const files = lodash_1.uniq(result.map(issue => issue.path));
            const actions = [];
            files.forEach(file => {
                actions.push(cb => {
                    correctFile({ file, ddfDataSet, result }, (correctFileError, content) => {
                        if (correctFileError) {
                            return cb(correctFileError);
                        }
                        this.jsonContent = content;
                        return jsonToCsv({ file, content, ddfDataSet }, (jsonToCsvError, csv) => {
                            if (jsonToCsvError) {
                                return cb(jsonToCsvError);
                            }
                            return cb(null, { file, csv });
                        });
                    });
                });
            });
            async_1.parallelLimit(actions, PROCESS_LIMIT, onJsonCorrected);
        });
    }
    write(fileDescriptors, onFilesSaved) {
        const actions = [];
        fileDescriptors.forEach(fileDescriptor => {
            actions.push(cb => file_1.backupFile(fileDescriptor.file, () => file_1.writeFile(fileDescriptor.file, fileDescriptor.csv, err => cb(err))));
        });
        async_1.parallelLimit(actions, PROCESS_LIMIT, onFilesSaved);
    }
}
exports.DdfJsonCorrector = DdfJsonCorrector;
