"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const async_1 = require("async");
const Mingo = require('mingo');
const COLLECTION_NAME = 'entities';
const PARALLEL_PROCESSES = 5;
class Entity {
    constructor(db) {
        this.db = db;
        this.collection = this.db.getCollection(COLLECTION_NAME);
        this.fileDescriptors = [];
    }
    getAllData() {
        return this.collection;
    }
    getDataByFiles() {
        return lodash_1.groupBy(this.getAllData(), record => record['$$source']);
    }
    getDataBy(condition) {
        const query = new Mingo.Query(condition);
        return query.find(this.collection);
    }
    addFileDescriptor(fileDescriptor) {
        this.fileDescriptors.push(fileDescriptor);
    }
    getTranslationsData(onTranslationsReady) {
        const transActions = this.fileDescriptors.map(fileDescriptor => fileDescriptor.getExistingTranslationDescriptors().map(translationDesc => onTranslationLoaded => {
            const transCollectionName = `translation-${translationDesc.file}`;
            this.db.fillCollection(transCollectionName, translationDesc.fullPath, err => {
                translationDesc.content = this.db.getCollection(transCollectionName);
                onTranslationLoaded(err);
            }, true);
        }));
        const transActionsNormalized = lodash_1.compact(lodash_1.flattenDeep(transActions));
        async_1.parallelLimit(transActionsNormalized, PARALLEL_PROCESSES, onTranslationsReady);
    }
}
exports.Entity = Entity;
