"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
function domainIsNotEntityDomain(domain) {
    return !domain || domain.concept_type !== ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN;
}
function domainIsNotEntitySetOrDomain(domain) {
    return domainIsNotEntityDomain(domain) && domain.concept_type !== ddf_things_1.CONCEPT_TYPE_ENTITY_SET;
}
function isIssueForEntitySet(domain, conceptRecord) {
    return conceptRecord.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_SET && domainIsNotEntityDomain(domain);
}
function isIssueForRole(domain, conceptRecord) {
    return conceptRecord.concept_type === ddf_things_1.CONCEPT_TYPE_ROLE && domainIsNotEntitySetOrDomain(domain);
}
function getIssueForConceptMandatoryField(ddfDataSet, conceptRecord) {
    const domain = ddfDataSet.getConcept().getRecordByKey(conceptRecord.domain);
    function isIssue() {
        return isIssueForEntitySet(domain, conceptRecord) || isIssueForRole(domain, conceptRecord);
    }
    if (lodash_1.isEmpty(conceptRecord.concept_type)) {
        return new issue_1.Issue(registry_1.CONCEPT_MANDATORY_FIELD_NOT_FOUND)
            .setPath(conceptRecord.$$source)
            .setData({
            line: conceptRecord.$$lineNumber,
            field: constants_1.CONCEPT_TYPE,
            value: JSON.stringify(conceptRecord)
        });
    }
    if (isIssue()) {
        return new issue_1.Issue(registry_1.CONCEPT_MANDATORY_FIELD_NOT_FOUND)
            .setPath(conceptRecord.$$source)
            .setData({
            line: conceptRecord.$$lineNumber,
            field: constants_1.DOMAIN_ID,
            value: domain
        });
    }
    return null;
}
exports.rule = {
    rule: (ddfDataSet) => ddfDataSet
        .getConcept()
        .getAllData()
        .filter(conceptRecord => !!conceptRecord.concept)
        .map(conceptRecord => getIssueForConceptMandatoryField(ddfDataSet, conceptRecord))
        .filter(issue => !!issue)
};
