"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const index_1 = require("../index");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
function hasConceptRelatedInvalidJSONIssue(ddfDataSet, concept) {
    const incorrectJSONFieldIssues = index_1.allRules[registry_1.INCORRECT_JSON_FIELD].rule(ddfDataSet);
    if (!lodash_1.isEmpty(incorrectJSONFieldIssues)) {
        const relatedIssue = incorrectJSONFieldIssues
            .filter(issue => !!issue.data)
            .find(issue => issue.data.line === concept.$$lineNumber + 1 && issue.path === concept.$$source);
        if (relatedIssue) {
            return true;
        }
    }
    return false;
}
function getDrillUpIssue(ddfDataSet, concept) {
    if (hasConceptRelatedInvalidJSONIssue(ddfDataSet, concept)) {
        return null;
    }
    if (concept.concept_type !== ddf_things_1.CONCEPT_TYPE_ENTITY_SET) {
        return new issue_1.Issue(registry_1.INVALID_DRILL_UP)
            .setPath(concept.$$source)
            .setData({
            line: concept.$$lineNumber + 1,
            type: concept.concept_type,
            reason: 'not an entity set or domain'
        });
    }
    const drillUps = JSON.parse(concept.drill_up);
    const reasons = drillUps
        .map(drillUpName => {
        const drillUpObject = ddfDataSet.getConcept().getAllData()
            .find(drillUpConceptName => drillUpConceptName.concept === drillUpName);
        let result = null;
        if (!drillUpObject) {
            result = { drillUpName, reason: 'Concept for Drillup is not found' };
        }
        if (drillUpObject && drillUpObject.domain !== concept.domain && drillUpObject.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_SET) {
            result = {
                conceptDomain: drillUpObject.domain,
                expectedDomain: concept.domain,
                reason: 'Domain in a Drillup is not a domain of a concept having this Drillup belongs to'
            };
        }
        if (drillUpObject && concept.domain !== drillUpObject.concept && drillUpObject.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN) {
            result = {
                conceptDomain: drillUpObject.concept,
                expectedDomain: concept.domain,
                reason: 'Entity domain in Drillup should be same as Entity domain for current Concept'
            };
        }
        return result;
    })
        .filter(reason => !!reason);
    if (!lodash_1.isEmpty(reasons)) {
        return new issue_1.Issue(registry_1.INVALID_DRILL_UP)
            .setPath(concept.$$source)
            .setData({
            line: concept.$$lineNumber + 1,
            reasons
        });
    }
    return null;
}
exports.rule = {
    rule: (ddfDataSet) => ddfDataSet.getConcept().getAllData()
        .filter(concept => concept.drill_up)
        .map(concept => getDrillUpIssue(ddfDataSet, concept))
        .filter(issue => !!issue)
};
