"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const issue_1 = require("../issue");
const registry_1 = require("../registry");
const data_package_1 = require("../../data/data-package");
const isDdfSchemaValid = (dataPackageContent) => {
    return dataPackageContent.ddfSchema && (dataPackageContent.ddfSchema.concepts ||
        dataPackageContent.ddfSchema.entities || dataPackageContent.ddfSchema.datapoints);
};
exports.rule = {
    rule: (ddfDataSet) => {
        const dataPackagePath = path.resolve(ddfDataSet.dataPackageDescriptor.rootFolder, data_package_1.DATA_PACKAGE_FILE);
        const consistencyDescriptor = ddfDataSet.dataPackageDescriptor.consistencyDescriptor;
        const dataPackageErrors = ddfDataSet.dataPackageDescriptor.errors;
        const issues = [];
        if (!lodash_1.isEmpty(dataPackageErrors)) {
            issues.push(new issue_1.Issue(registry_1.INCONSISTENT_DATAPACKAGE).setPath(dataPackagePath).setData(dataPackageErrors));
        }
        if (!consistencyDescriptor.valid) {
            const errors = consistencyDescriptor.errors.map(error => error.message);
            issues.push(new issue_1.Issue(registry_1.INCONSISTENT_DATAPACKAGE).setPath(dataPackagePath).setData(errors));
        }
        if (!isDdfSchemaValid(ddfDataSet.dataPackageDescriptor.getDataPackageObject())) {
            issues.push(new issue_1.Issue(registry_1.INCONSISTENT_DATAPACKAGE)
                .setPath(dataPackagePath).setData({ reason: 'ddfSchema section is missing or invalid' }));
        }
        return issues;
    }
};
