"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
exports.rule = {
    rule: (ddfDataSet) => {
        return lodash_1.compact(lodash_1.flattenDeep(ddfDataSet.getDataPackageResources().map(resource => {
            const dataPackageHeaders = resource.schema.fields.map(field => field.name);
            const relatedFileDescriptor = lodash_1.head(ddfDataSet.dataPackageDescriptor.fileDescriptors.filter(fileDescriptor => fileDescriptor.fullPath === path.resolve(fileDescriptor.rootFolder, resource.path)));
            if (!relatedFileDescriptor) {
                return null;
            }
            const realHeaders = relatedFileDescriptor.headers;
            const headersDifference = lodash_1.difference(dataPackageHeaders, realHeaders);
            let issue = null;
            if (!lodash_1.isEmpty(headersDifference)) {
                issue = new issue_1.Issue(registry_1.DATAPACKAGE_INCORRECT_FIELDS)
                    .setPath(relatedFileDescriptor.fullPath)
                    .setData({ dataPackageHeaders, realHeaders, headersDifference });
            }
            return issue;
        })));
    }
};
