"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const issue_1 = require("../issue");
const registry_1 = require("../../ddf-rules/registry");
const NON_PRINTABLE_CHARACTERS_REGEXP = /[\x00-\x1F]+/g;
const getSuggestions = (filename) => [filename.replace(NON_PRINTABLE_CHARACTERS_REGEXP, '')];
const getIssuesByFileDescriptors = (ddfDataSet) => {
    const result = ddfDataSet.wrongFileDescriptors
        .map((fileDescriptor) => fileDescriptor.issues.map(issue => new issue_1.Issue(issue.type).setPath(issue.fullPath).setData(issue.data)));
    return lodash_1.compact(lodash_1.flattenDeep(result));
};
const getIssuesByDataPackage = (ddfDataSet) => {
    const dataPackageDescriptor = ddfDataSet.dataPackageDescriptor;
    const result = dataPackageDescriptor.fileDescriptors
        .filter((fileDescriptor) => NON_PRINTABLE_CHARACTERS_REGEXP.test(fileDescriptor.fullPath))
        .map((fileDescriptor) => new issue_1.Issue(registry_1.INCORRECT_FILE)
        .setPath(fileDescriptor.fullPath)
        .setData({ reason: 'Non printable characters in filename' })
        .setSuggestions(getSuggestions(fileDescriptor.fullPath)));
    return lodash_1.compact(lodash_1.flattenDeep(result));
};
exports.rule = {
    rule: (ddfDataSet) => lodash_1.concat(getIssuesByFileDescriptors(ddfDataSet), getIssuesByDataPackage(ddfDataSet))
};
