"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const path_1 = require("path");
const registry_1 = require("../registry");
const constants_1 = require("../../ddf-definitions/constants");
const issue_1 = require("../issue");
exports.rule = {
    rule: (ddfDataSet) => lodash_1.compact(lodash_1.flattenDeep(ddfDataSet.getDataPackageResources().map(resource => {
        const dataPackageFields = resource.schema.fields.map(field => field.name.replace(/^is--/, ''));
        const allConceptsIds = ddfDataSet.getConcept().getIds().concat(constants_1.PREDEFINED_CONCEPTS);
        const nonConcepts = dataPackageFields.filter(header => !lodash_1.includes(allConceptsIds, header));
        let issue = null;
        if (lodash_1.isEmpty(dataPackageFields)) {
            issue = new issue_1.Issue(registry_1.DATAPACKAGE_NON_CONCEPT_FIELD)
                .setPath(path_1.resolve(ddfDataSet.rootPath, resource.path))
                .setData({ reason: 'fields section in datapackage is empty' });
        }
        if (!lodash_1.isEmpty(nonConcepts) && !issue) {
            issue = new issue_1.Issue(registry_1.DATAPACKAGE_NON_CONCEPT_FIELD)
                .setPath(path_1.resolve(ddfDataSet.rootPath, resource.path))
                .setData({ nonConcepts });
        }
        return issue;
    })))
};
