"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const issue_1 = require("../issue");
const registry_1 = require("../registry");
const data_package_1 = require("../../data/data-package");
const path = require("path");
const constants_1 = require("../../ddf-definitions/constants");
const toArray = value => lodash_1.isArray(value) ? value : [value];
const fillConceptsSetBySchema = (dataPackageSchema, conceptsSet) => {
    [
        ...dataPackageSchema.concepts,
        ...dataPackageSchema.entities,
        ...dataPackageSchema.datapoints
    ].forEach(resource => {
        for (const pk of toArray(resource.primaryKey)) {
            conceptsSet.add(pk);
        }
        conceptsSet.add(resource.value);
    });
};
const fillResources = (ddfDataSet, conceptsSet) => {
    ddfDataSet.getDataPackageResources().forEach(resource => {
        for (const pk of toArray(resource.schema.primaryKey)) {
            conceptsSet.add(pk);
        }
        for (const field of resource.schema.fields) {
            conceptsSet.add(field.name);
        }
    });
};
exports.rule = {
    rule: (ddfDataSet) => {
        const dataPackagePath = path.resolve(ddfDataSet.dataPackageDescriptor.rootFolder, data_package_1.DATA_PACKAGE_FILE);
        const conceptsSet = new Set();
        const dataPackageSchema = ddfDataSet.getDataPackageSchema();
        if (dataPackageSchema) {
            fillConceptsSetBySchema(dataPackageSchema, conceptsSet);
        }
        fillResources(ddfDataSet, conceptsSet);
        const originalConcepts = ddfDataSet.getConcept().getAllData().map(record => record.concept);
        return lodash_1.compact(Array.from(conceptsSet.values()))
            .map(concept => concept.replace(/^is--/, ''))
            .filter(concept => concept !== constants_1.CONCEPT_ID && concept !== constants_1.CONCEPT_TYPE && !lodash_1.includes(originalConcepts, concept))
            .map(concept => new issue_1.Issue(registry_1.DATAPACKAGE_NONEXISTENT_CONCEPT).setPath(dataPackagePath).setData(concept));
    }
};
