"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const issue_1 = require("../issue");
const registry_1 = require("../registry");
const data_package_1 = require("../../data/data-package");
const getNonexistentResourcesIssues = (ddfDataSet, dataPackagePath, resourcesMap) => {
    if (!ddfDataSet.getDataPackageSchema()) {
        return [];
    }
    return lodash_1.compact([
        ...(ddfDataSet.getDataPackageSchema().concepts || []),
        ...(ddfDataSet.getDataPackageSchema().entities || []),
        ...(ddfDataSet.getDataPackageSchema().datapoints || []),
        ...(ddfDataSet.getDataPackageSchema().synonyms || []),
    ].map(record => {
        const nonexistentResources = record.resources.filter(resource => !resourcesMap.has(resource));
        if (!lodash_1.isEmpty(nonexistentResources)) {
            return new issue_1.Issue(registry_1.DATAPACKAGE_NONEXISTENT_RESOURCE)
                .setPath(dataPackagePath)
                .setData({
                nonexistentResources, record,
                specific: 'is NOT found in resources, but found in schema section'
            });
        }
        return null;
    }));
};
const fillResourceMapCounters = (ddfDataSet, resourcesMap) => {
    if (!ddfDataSet.getDataPackageSchema()) {
        return [];
    }
    [
        ...(ddfDataSet.getDataPackageSchema().concepts || []),
        ...(ddfDataSet.getDataPackageSchema().entities || []),
        ...(ddfDataSet.getDataPackageSchema().datapoints || []),
        ...(ddfDataSet.getDataPackageSchema().synonyms || [])
    ].forEach(record => {
        record.resources.forEach(resource => {
            if (resourcesMap.has(resource)) {
                resourcesMap.set(resource, resourcesMap.get(resource) + 1);
            }
            return resource;
        });
    });
};
const getNonexistentSchemaResourcesIssues = (dataPackagePath, resourcesMap) => Array.from(resourcesMap.keys())
    .filter(resource => resourcesMap.get(resource) === 0)
    .map(resource => new issue_1.Issue(registry_1.DATAPACKAGE_NONEXISTENT_RESOURCE)
    .setPath(dataPackagePath)
    .setData({ resource, specific: 'is NOT found in ddfSchema schema, but found in resources section' }));
exports.rule = {
    rule: (ddfDataSet) => {
        const dataPackagePath = path.resolve(ddfDataSet.dataPackageDescriptor.rootFolder, data_package_1.DATA_PACKAGE_FILE);
        const resourcesMap = ddfDataSet.getDataPackageResources()
            .map(resource => resource.name)
            .reduce((mapValue, resourceName) => {
            mapValue.set(resourceName, 0);
            return mapValue;
        }, new Map());
        fillResourceMapCounters(ddfDataSet, resourcesMap);
        return [
            ...getNonexistentResourcesIssues(ddfDataSet, dataPackagePath, resourcesMap),
            ...getNonexistentSchemaResourcesIssues(dataPackagePath, resourcesMap)
        ];
    }
};
