"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
let cache;
exports.resetCache = () => {
    cache = {};
};
exports.resetCache();
exports.cacheFor = {
    conceptTypeDictionary: dataPointDescriptor => {
        const key = `conceptTypeDictionary@${dataPointDescriptor.fileDescriptor.file}`;
        if (!cache[key]) {
            cache[key] = dataPointDescriptor.ddfDataSet.getConcept()
                .getDictionary(dataPointDescriptor.fileDescriptor.headers, constants_1.CONCEPT_TYPE);
        }
        return cache[key];
    },
    keysByType: (dataPointDescriptor, type) => {
        const key = `${type}Keys@${dataPointDescriptor.fileDescriptor.file}`;
        if (!cache[key]) {
            const conceptTypeDictionary = exports.cacheFor.conceptTypeDictionary(dataPointDescriptor);
            cache[key] = Object.keys(conceptTypeDictionary)
                .filter(conceptTypeKey => conceptTypeDictionary[conceptTypeKey] === type);
        }
        return cache[key];
    },
    constraintsByFileDescriptor: (dataPointDescriptor) => {
        const forExpectedFile = (resourceRecord) => dataPointDescriptor.fileDescriptor.file === resourceRecord.path;
        const hasConstraints = (field) => field.constraints;
        const getSchemaFields = (resourceRecord) => resourceRecord.schema && resourceRecord.schema.fields ? resourceRecord.schema.fields : [];
        const key = `${dataPointDescriptor.ddfDataSet.rootPath}@Constraints@${dataPointDescriptor.fileDescriptor.file}`;
        if (!cache[key]) {
            const dataPackageDescriptor = dataPointDescriptor.ddfDataSet.getDataPackageDescriptor();
            const res = dataPackageDescriptor.getDataPackageObject().resources;
            cache[key] = lodash_1.compact(lodash_1.flattenDeep(res.filter(forExpectedFile).map((resourceRecord) => getSchemaFields(resourceRecord).filter(hasConstraints).map((field) => ({
                fullPath: dataPointDescriptor.fileDescriptor.fullPath,
                file: dataPointDescriptor.fileDescriptor.file,
                fieldName: field.name,
                constraints: field.constraints.enum
            })))));
        }
        return cache[key];
    },
    getEntitiesByRecord: dataPointDescriptor => {
        const key = `recordKeysHash@${dataPointDescriptor.fileDescriptor.file}`;
        if (!cache[key]) {
            const conceptTypeDictionary = exports.cacheFor.conceptTypeDictionary(dataPointDescriptor);
            const entities = lodash_1.keys(dataPointDescriptor.record).filter(key => conceptTypeDictionary[key] === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN || conceptTypeDictionary[key] === ddf_things_1.CONCEPT_TYPE_ENTITY_SET);
            cache[key] = entities;
        }
        return cache[key];
    },
    getEntityValueHash: dataPointDescriptor => {
        const key = `entityValueHash`;
        if (!cache[key]) {
            const result = {};
            const conceptTypeHash = {};
            const conceptDomainHash = {};
            const conceptsContent = dataPointDescriptor.ddfDataSet.getConcept().getAllData();
            const entitiesContent = dataPointDescriptor.ddfDataSet.getEntity().getAllData();
            for (let conceptRecord of conceptsContent) {
                conceptTypeHash[conceptRecord.concept] = conceptRecord.concept_type;
                conceptDomainHash[conceptRecord.concept] = conceptRecord.domain;
            }
            for (let entityRecord of entitiesContent) {
                const isEntityWithIsSign = {};
                const recordKeys = lodash_1.keys(entityRecord);
                const looksLikeEntity = key => !ddf_things_1.looksLikeIsField(key) || (ddf_things_1.looksLikeIsField(key) && ddf_things_1.isDdfTrue(entityRecord[key]));
                const isEntity = key => conceptTypeHash[key] === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN || conceptTypeHash[key] === ddf_things_1.CONCEPT_TYPE_ENTITY_SET;
                const entitiesSet = lodash_1.reduce(recordKeys, (result, key) => {
                    if (looksLikeEntity(key)) {
                        const newKey = key.replace(/is--/, '');
                        if (ddf_things_1.looksLikeIsField(key)) {
                            isEntityWithIsSign[newKey] = true;
                        }
                        if (isEntity(newKey)) {
                            result.add(newKey);
                        }
                        if (conceptDomainHash[newKey]) {
                            result.add(conceptDomainHash[newKey]);
                        }
                    }
                    return result;
                }, new Set());
                const entities = Array.from(entitiesSet);
                for (let plainEntity of entities) {
                    const entityKey = `${plainEntity}@${entityRecord[plainEntity]}`;
                    result[entityKey] = entities;
                    if (conceptDomainHash[plainEntity]) {
                        const entityDomainKey = `${conceptDomainHash[plainEntity]}@${entityRecord[plainEntity]}`;
                        result[entityDomainKey] = entities;
                    }
                    for (let additionalEntityKey of lodash_1.keys(isEntityWithIsSign)) {
                        const entityKeyWithIsSign = `${additionalEntityKey}@${entityRecord[plainEntity]}`;
                        result[entityKeyWithIsSign] = entities;
                    }
                }
            }
            cache[key] = result;
        }
        return cache[key];
    }
};
