"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
const getBooleanValues = (values) => values.filter(value => ddf_things_1.isDdfBoolean(value));
exports.rule = {
    rule: (ddfDataSet) => {
        const typesHash = ddfDataSet.getConcept().getDictionary(null, constants_1.CONCEPT_TYPE);
        const booleanConcepts = [];
        const issues = [];
        for (const conceptName of lodash_1.keys(typesHash)) {
            if (typesHash[conceptName] === ddf_things_1.CONCEPT_TYPE_BOOLEAN) {
                booleanConcepts.push(conceptName);
            }
        }
        const entities = ddfDataSet.getEntity().getAllData();
        const valuesHash = {};
        for (const entityRecord of entities) {
            for (const concept of lodash_1.keys(entityRecord)) {
                if (!lodash_1.includes(booleanConcepts, concept) && !ddf_things_1.looksLikeIsField(concept)) {
                    if (!valuesHash[concept]) {
                        valuesHash[concept] = [];
                    }
                    valuesHash[concept].push(entityRecord[concept]);
                }
            }
        }
        for (const concept of lodash_1.keys(valuesHash)) {
            valuesHash[concept] = lodash_1.uniq(lodash_1.compact(valuesHash[concept]));
            const booleanValues = getBooleanValues(valuesHash[concept]);
            if (booleanValues.length === valuesHash[concept].length && valuesHash[concept].length > 0) {
                issues.push(new issue_1.Issue(registry_1.CONCEPT_LOOKS_LIKE_BOOLEAN).setData({ concept }));
            }
        }
        return issues;
    }
};
