"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
const getGidByResource = (ddfDataSet, entitiesPath) => {
    const parsedEntitiesPath = path.parse(entitiesPath);
    const relativeDdfPath = path.relative(ddfDataSet.dataPackageDescriptor.rootFolder, parsedEntitiesPath.dir);
    const dataPackageCompatiblePath = path.join(relativeDdfPath, parsedEntitiesPath.base);
    const resource = ddfDataSet.getDataPackageResources().filter(record => record.path === dataPackageCompatiblePath && constants_1.getTypeByPrimaryKey(record.schema.primaryKey) === constants_1.ENTITY);
    return lodash_1.head(resource).schema.primaryKey;
};
exports.rule = {
    rule: (ddfDataSet) => {
        const entityConcepts = ddfDataSet.getConceptsByType(ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN, ddf_things_1.CONCEPT_TYPE_ENTITY_SET);
        const entitiesByFile = ddfDataSet.getEntity().getDataByFiles();
        const issues = [];
        for (const entityFileName of lodash_1.keys(entitiesByFile)) {
            const gid = getGidByResource(ddfDataSet, entityFileName);
            for (const record of entitiesByFile[entityFileName]) {
                if (lodash_1.includes(entityConcepts, record[gid])) {
                    issues.push(new issue_1.Issue(registry_1.ENTITY_VALUE_AS_ENTITY_NAME)
                        .setPath(record.$$source)
                        .setData({
                        entityName: gid,
                        entityRecord: record
                    }));
                }
            }
        }
        return issues;
    }
};
