"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
exports.rule = {
    rule: (ddfDataSet) => {
        const typesHash = ddfDataSet.getConcept().getDictionary(null, constants_1.CONCEPT_TYPE);
        const booleanConcepts = [];
        const issues = [];
        for (const conceptName of lodash_1.keys(typesHash)) {
            if (typesHash[conceptName] === ddf_things_1.CONCEPT_TYPE_BOOLEAN) {
                booleanConcepts.push(conceptName);
            }
        }
        const entities = ddfDataSet.getEntity().getAllData();
        for (const entityRecord of entities) {
            const fieldsWithWrongValues = [];
            for (const booleanConcept of booleanConcepts) {
                if (entityRecord[booleanConcept]) {
                    if (!ddf_things_1.isDdfBoolean(entityRecord[booleanConcept])) {
                        fieldsWithWrongValues.push(booleanConcept);
                    }
                }
            }
            if (!lodash_1.isEmpty(fieldsWithWrongValues)) {
                const issue = new issue_1.Issue(registry_1.INCORRECT_BOOLEAN_ENTITY)
                    .setPath(entityRecord.$$source)
                    .setData({
                    record: entityRecord,
                    fieldsWithWrongValues,
                    line: entityRecord.$$lineNumber
                });
                issues.push(issue);
            }
        }
        return issues;
    }
};
