"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const constants_1 = require("../../ddf-definitions/constants");
const issue_1 = require("../issue");
const checkConstraintValue = (ddfDataSet, name, value) => {
    const allEntities = ddfDataSet.getEntity().getAllData();
    const domainHash = ddfDataSet.getConcept().getDictionary(null, constants_1.DOMAIN_ID);
    for (const record of allEntities) {
        if (record[name] === value || record[domainHash[name]] === value) {
            return true;
        }
    }
    return false;
};
const forDataPointType = (fileDescriptor) => fileDescriptor.type === constants_1.DATA_POINT;
const constraintValueAreNotPresentInEntities = (ddfDataSet, name, value) => !checkConstraintValue(ddfDataSet, name, value);
const getConstraintsByField = (field) => lodash_1.get(field, 'constraints.enum', []);
exports.rule = {
    rule: (ddfDataSet) => {
        const dataPackageObject = ddfDataSet.dataPackageDescriptor;
        const issues = dataPackageObject.fileDescriptors.filter(forDataPointType).map((fileDescriptor) => {
            const relatedResourceSchema = dataPackageObject.getDataPackageObject().resources
                .find(resource => path.resolve(ddfDataSet.rootPath, resource.path) === fileDescriptor.fullPath);
            const constrainedSchemaFields = relatedResourceSchema.schema.fields.filter(field => field.constraints && field.constraints.enum);
            return constrainedSchemaFields.map((field) => {
                const issuesSource = getConstraintsByField(field).filter(value => constraintValueAreNotPresentInEntities(ddfDataSet, field.name, value));
                return issuesSource.map(value => new issue_1.Issue(registry_1.UNEXISTING_CONSTRAINT_VALUE).setPath(fileDescriptor.fullPath).setData({ constraintEntityValue: value }));
            });
        });
        return lodash_1.compact(lodash_1.flattenDeep(issues));
    }
};
