"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const getPrimaryKeyByFile = (ddfDataSet, file) => lodash_1.head(ddfDataSet.getDataPackageResources()
    .filter(resource => resource.path === file)
    .map(resource => resource.schema.primaryKey));
exports.rule = {
    rule: (ddfDataSet) => {
        const issues = [];
        const synonymsByFile = ddfDataSet.getSynonym().getDataByFiles();
        const duplicationsHash = new Map();
        const filesHash = new Map();
        const pkHash = new Map();
        for (const synonymsFile of lodash_1.keys(synonymsByFile)) {
            const primaryKey = getPrimaryKeyByFile(ddfDataSet, path.relative(ddfDataSet.rootPath, synonymsFile)).sort();
            for (const record of synonymsByFile[synonymsFile]) {
                let pkData = '';
                for (const pkPart of primaryKey) {
                    pkData += `${record[pkPart]}#`;
                }
                if (!duplicationsHash.has(pkData)) {
                    duplicationsHash.set(pkData, 1);
                    filesHash.set(pkData, synonymsFile);
                    pkHash.set(pkData, pkData.substring(0, pkData.length - 1).split('#'));
                }
                else {
                    duplicationsHash.set(pkData, duplicationsHash.get(pkData) + 1);
                }
            }
        }
        for (const pkData of duplicationsHash.keys()) {
            if (duplicationsHash.get(pkData) > 1) {
                issues.push(new issue_1.Issue(registry_1.DUPLICATED_SYNONYM_KEY).setPath(filesHash.get(pkData)).setData(pkHash.get(pkData)));
            }
        }
        return issues;
    }
};
