"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const NOT_ALPHANUMERIC = /[^a-z0-9_]/;
function getIdentifierColumnsNames(ddfDataSet, firstRecord) {
    const concept = ddfDataSet.getConcept();
    const relatedConceptNames = concept.getDataIdsByType(ddf_things_1.CONCEPT_TYPE_ENTITY_SET)
        .concat(concept.getDataIdsByType(ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN));
    return lodash_1.keys(firstRecord)
        .filter(entityHeader => lodash_1.includes(relatedConceptNames, entityHeader));
}
function getIncorrectConceptIdentifierIssues(ddfDataSet) {
    return ddfDataSet.getConcept()
        .getAllData()
        .filter(conceptRecord => conceptRecord.concept)
        .filter(conceptRecord => NOT_ALPHANUMERIC.test(conceptRecord.concept))
        .map(conceptRecord => new issue_1.Issue(registry_1.INCORRECT_IDENTIFIER)
        .setPath(conceptRecord.$$source)
        .setData([{
            conceptValue: conceptRecord.concept,
            line: conceptRecord.$$lineNumber
        }]));
}
function getIncorrectEntityIdentifierIssues(ddfDataSet) {
    const entitiesByFile = ddfDataSet.getEntity().getDataByFiles();
    function getIssueData(entityFileName, entitiesConcepts) {
        return entitiesByFile[entityFileName]
            .map(entityRecord => entitiesConcepts
            .filter(entityConcept => !!entityRecord[entityConcept])
            .filter(entityConcept => NOT_ALPHANUMERIC.test(entityRecord[entityConcept]))
            .map(entityConcept => ({
            conceptName: entityConcept,
            conceptValue: entityRecord[entityConcept],
            line: entityRecord.$$lineNumber
        })));
    }
    function getIssueByEntityFile(entityFileName) {
        const entitiesConcepts = getIdentifierColumnsNames(ddfDataSet, lodash_1.head(entitiesByFile[entityFileName]));
        const issueData = lodash_1.compact(lodash_1.flattenDeep(getIssueData(entityFileName, entitiesConcepts)));
        if (!lodash_1.isEmpty(issueData)) {
            return new issue_1.Issue(registry_1.INCORRECT_IDENTIFIER)
                .setPath(entityFileName)
                .setData(issueData);
        }
        return null;
    }
    const issues = lodash_1.keys(entitiesByFile)
        .map(entityFileName => getIssueByEntityFile(entityFileName));
    return lodash_1.compact(issues);
}
exports.rule = {
    rule: (ddfDataSet) => lodash_1.concat(getIncorrectEntityIdentifierIssues(ddfDataSet), getIncorrectConceptIdentifierIssues(ddfDataSet))
};
