"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
function isJsonLike(value) {
    return /^\[.*\]$|^{.*}$/g.test(value);
}
function detectColumnsHavingJson(ddfDataWrapper) {
    const dataByFiles = lodash_1.extend({}, ddfDataWrapper.getDataByFiles());
    const serviceFields = ['$$source', '$$lineNumber'];
    const result = {};
    Object.keys(dataByFiles).forEach(fileName => {
        const columnNames = Object.keys(lodash_1.head(dataByFiles[fileName])).filter(key => !lodash_1.includes(serviceFields, key));
        const jsonColumns = [];
        columnNames.forEach(column => {
            let isNotLikeJson = false;
            dataByFiles[fileName].forEach(record => {
                if (!isJsonLike(record[column]) && record[column]) {
                    isNotLikeJson = true;
                }
            });
            if (!isNotLikeJson) {
                jsonColumns.push(column);
            }
        });
        result[fileName] = jsonColumns;
    });
    return result;
}
function correctBoolean(json) {
    return json.replace(/true|false/ig, booleanValue => booleanValue.toLowerCase());
}
function ensureJsonIsSafe(json) {
    return json.replace(/\)[;\n]/g, '');
}
function correctJSON(wrongJSON) {
    let result = null;
    try {
        const convertFun = new Function(`return JSON.stringify(${ensureJsonIsSafe(correctBoolean(wrongJSON))})`);
        result = convertFun();
    }
    catch (ex) {
        result = null;
    }
    return result;
}
function isValidJSON(json) {
    let result = true;
    try {
        JSON.parse(json);
    }
    catch (ex) {
        result = false;
    }
    return result;
}
function isCorrectionPossible(record, column, fileName) {
    return record[column] && record.$$source === fileName && !isValidJSON(record[column]);
}
function getIncorrectJsonIssues(ddfDataWrapper) {
    const columnsHavingJson = detectColumnsHavingJson(ddfDataWrapper);
    const issues = Object.keys(columnsHavingJson).map(fileName => {
        return columnsHavingJson[fileName].map(jsonColumn => {
            return ddfDataWrapper.getAllData()
                .filter(record => isCorrectionPossible(record, jsonColumn, fileName))
                .map(record => {
                const data = {
                    column: jsonColumn,
                    line: record.$$lineNumber + 1,
                    value: record[jsonColumn]
                };
                const correctedJSON = correctJSON(record[jsonColumn]);
                const issue = new issue_1.Issue(registry_1.INCORRECT_JSON_FIELD)
                    .setPath(record.$$source)
                    .setData(data);
                if (correctedJSON) {
                    issue.setSuggestions([correctedJSON]);
                }
                return issue;
            });
        });
    });
    return lodash_1.compact(lodash_1.flattenDeep(issues));
}
exports.rule = {
    rule: (ddfDataSet) => getIncorrectJsonIssues(ddfDataSet.getConcept()).concat(getIncorrectJsonIssues(ddfDataSet.getEntity()))
};
