"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
exports.rule = {
    rule: (ddfDataSet) => {
        const issues = [];
        const expectedConcepts = ddfDataSet.getConcept().getAllData().filter(conceptRecord => conceptRecord.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN ||
            conceptRecord.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_SET ||
            conceptRecord.concept_type === ddf_things_1.CONCEPT_TYPE_TIME).map(conceptRecord => conceptRecord.concept);
        ddfDataSet.getDataPoint().fileDescriptors.forEach((fileDescriptor, index) => {
            const dataPackage = ddfDataSet.getDataPoint().dataPackageObjects[index].dataPackageContent;
            const expectedDataPackageResource = dataPackage.resources
                .find(resource => resource.path === fileDescriptor.file);
            const primaryKeyInvalidParts = expectedDataPackageResource.schema.primaryKey
                .filter(partOfPrimaryKey => !lodash_1.includes(expectedConcepts, partOfPrimaryKey));
            const sharedHeaders = lodash_1.intersection(fileDescriptor.headers, expectedDataPackageResource.schema.primaryKey);
            if (!lodash_1.isEmpty(primaryKeyInvalidParts)) {
                issues.push(new issue_1.Issue(registry_1.WRONG_DATA_POINT_HEADER)
                    .setPath(fileDescriptor.fullPath)
                    .setData({ primaryKeyInvalidParts, reason: 'some parts of the primary key have an incorrect type' }));
            }
            if (sharedHeaders.length !== expectedDataPackageResource.schema.primaryKey.length) {
                issues.push(new issue_1.Issue(registry_1.WRONG_DATA_POINT_HEADER)
                    .setPath(fileDescriptor.fullPath)
                    .setData({
                    sharedHeaders,
                    primaryKey: expectedDataPackageResource.schema.primaryKey,
                    reason: 'primary from datapackage does not correspond with header from ddf file'
                }));
            }
        });
        return issues;
    }
};
