"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("./registry");
class Issue {
    constructor(type) {
        this.type = type;
        this.suggestions = [];
    }
    setPath(path) {
        this.path = path;
        return this;
    }
    setData(data) {
        this.data = data;
        return this;
    }
    setSuggestions(suggestions) {
        if (suggestions) {
            this.suggestions = suggestions;
        }
        return this;
    }
    view() {
        const result = {
            id: Symbol.keyFor(this.type),
            type: registry_1.descriptions[this.type],
            howToFix: registry_1.howToFix[this.type],
            path: this.path,
            data: this.data,
            isWarning: lodash_1.includes(registry_1.tags[this.type], registry_1.WARNING_TAG)
        };
        if (this.suggestions && this.suggestions.length > 0) {
            result['suggestions'] = this.suggestions;
        }
        if (!lodash_1.isEmpty(registry_1.tags[this.type])) {
            result['tags'] = registry_1.tags[this.type].map(tag => Symbol.keyFor(tag));
        }
        return result;
    }
}
exports.Issue = Issue;
