"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
exports.rule = {
    isTranslation: true,
    getStorageTemplate: () => ({
        hash: new Set(),
        duplicatedHashes: [],
        content: {}
    }),
    aggregateRecord: (dataPointDescriptor, storage) => {
        const sortedPrimaryKey = dataPointDescriptor.fileDescriptor.primaryKey.sort();
        const dimensionData = sortedPrimaryKey.map(keyPart => `${keyPart}:${dataPointDescriptor.record[keyPart]}`).join(',');
        const indicatorName = lodash_1.difference(dataPointDescriptor.fileDescriptor.headers, dataPointDescriptor.fileDescriptor.primaryKey).join(',');
        const recordHash = `${dimensionData}@${indicatorName}#${dataPointDescriptor.fileDescriptor.translationId}`;
        if (storage.hash.has(recordHash)) {
            storage.duplicatedHashes.push(recordHash);
        }
        storage.hash.add(recordHash);
        if (!storage.content[recordHash]) {
            storage.content[recordHash] = [];
        }
        storage.content[recordHash].push({
            file: dataPointDescriptor.fileDescriptor.file,
            record: dataPointDescriptor.record,
            line: dataPointDescriptor.line
        });
    },
    aggregativeRule: (storage) => {
        const duplicates = lodash_1.uniq(storage.duplicatedHashes);
        const issues = [];
        for (const hash of duplicates) {
            issues.push(new issue_1.Issue(registry_1.DUPLICATED_DATA_POINT_TRANSLATION_KEY).setData(storage.content[hash]));
        }
        return issues;
    }
};
