"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
function getAsArray(value) {
    return lodash_1.isArray(value) ? value : [value];
}
exports.rule = {
    isTranslation: true,
    rule: (ddfDataSet) => lodash_1.compact(lodash_1.flattenDeep(ddfDataSet.fileDescriptors.map(fileDescriptor => fileDescriptor.getExistingTranslationDescriptors().map(translationDescriptor => {
        const headersDiff = lodash_1.intersection(fileDescriptor.headers, translationDescriptor.headers);
        const primaryKey = getAsArray(fileDescriptor.primaryKey);
        const primaryKeyDiff = lodash_1.intersection(primaryKey, translationDescriptor.headers);
        const issues = [];
        const pushIssue = issueData => {
            issues.push(new issue_1.Issue(registry_1.UNEXPECTED_TRANSLATION_HEADER)
                .setPath(translationDescriptor.fullPath)
                .setData(issueData));
        };
        if (translationDescriptor.headers.length > headersDiff.length || headersDiff.length === 0) {
            pushIssue({
                reason: 'extra data in translation',
                ddfFileHeaders: fileDescriptor.headers,
                translationHeaders: translationDescriptor.headers
            });
        }
        if (primaryKeyDiff.length < primaryKey.length) {
            pushIssue({
                reason: 'non consistent primary key',
                primaryKey: fileDescriptor.primaryKey,
                translationHeaders: translationDescriptor.headers
            });
        }
        return issues;
    }))))
};
