/// <reference types="node" />
import { EventEmitter } from 'events';
import { DdfDataSet } from './ddf-definitions/ddf-data-set';
import { IssuesFilter } from './utils/issues-filter';
export interface IDdfValidationSummary {
    errors: number;
    warnings: number;
    errorsByFiles: {
        [file: string]: number;
    };
    errorsByRuleId: {
        [ruleId: string]: number;
    };
    warningsByFiles: {
        [file: string]: number;
    };
    warningsByRuleId: {
        [ruleId: string]: number;
    };
}
export declare class ValidatorBase {
    protected messageEmitter: EventEmitter;
    constructor();
    sendMessage(data: any): void;
    onMessage(data: any): EventEmitter;
    abandon(): void;
    isAbandoned(): boolean;
}
export declare class JSONValidator extends ValidatorBase {
    rootPath: string;
    settings: any;
    issueEmitter: EventEmitter;
    issuesFilter: IssuesFilter;
    ddfDataSet: DdfDataSet;
    summary: IDdfValidationSummary;
    private errorsSummaryByFileHash;
    private errorsSummaryByRuleHash;
    private warningsSummaryByFileHash;
    private warningsSummaryByRuleHash;
    constructor(rootPath: any, settings: any);
    on(type: any, data: any): EventEmitter;
    validate(): void;
    private initSummary();
    private fillSummary(issue);
}
export declare class StreamValidator extends ValidatorBase {
    rootPath: string;
    settings: any;
    issueEmitter: EventEmitter;
    issuesFilter: IssuesFilter;
    ddfDataSet: DdfDataSet;
    constructor(rootPath: any, settings: any);
    on(type: any, data: any): EventEmitter;
    multiThreadProcessing(): void;
    validate(): void;
}
export declare class SimpleValidator extends ValidatorBase {
    rootPath: string;
    settings: any;
    issueEmitter: EventEmitter;
    issuesFilter: IssuesFilter;
    ddfDataSet: DdfDataSet;
    isDataSetCorrect: boolean;
    constructor(rootPath: any, settings: any);
    on(type: any, data: any): EventEmitter;
    validate(): void;
}
export interface IDataPackageInfo {
    ddfPath: string;
    dataPackagePath: string;
    exists: boolean;
}
export declare function getDataPackageInfo(ddfRootFolder: string): IDataPackageInfo;
export interface IDataPackageCreationParameters {
    ddfRootFolder: string;
    newDataPackagePriority?: boolean;
    externalSettings?: any;
}
export declare function createDataPackage(parameters: IDataPackageCreationParameters, onNotice: Function, onDataPackageReady: Function): any;
export declare function getDataPackageActuality(parameters: IDataPackageCreationParameters, cb: Function): any;
export declare const validate: (validator: any) => void;
