"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const async_1 = require("async");
const process_one_data_points_chunk_1 = require("../stories/process-one-data-points-chunk");
const shared_1 = require("../shared");
class DataPointChunksProcessingStory {
    constructor(fileDescriptorsChunks, issueEmitter) {
        this.fileDescriptorsChunks = fileDescriptorsChunks;
        this.issueEmitter = issueEmitter;
    }
    withCustomRules(customRules) {
        this.customRules = customRules;
        return this;
    }
    processDataPointChunks(ddfDataSet, issuesFilter) {
        this.processedIssues = this.totalIssues = 0;
        const actions = this.fileDescriptorsChunks.map(fileDescriptors => onFileCompleteProcessed => {
            if (shared_1.supervisor.abandon) {
                return onFileCompleteProcessed(new Error('abandoned by the external reason'), null);
            }
            const dataPointsCheckingStory = new process_one_data_points_chunk_1.ProcessOneDataPointsChunkStory(fileDescriptors, this.issueEmitter);
            dataPointsCheckingStory.withCustomRules(this.customRules).collect(ddfDataSet, issuesFilter, () => {
                dataPointsCheckingStory.analyze();
                this.issueEmitter.emit('chunk-progress');
                onFileCompleteProcessed(shared_1.supervisor.abandon ? new Error('abandoned by the external reason') : null, dataPointsCheckingStory.getTotal());
            });
        });
        this.issueEmitter.emit('init-chunk-progress', actions.length);
        async_1.parallelLimit(actions, shared_1.CONCURRENT_OPERATIONS_AMOUNT, (err, results) => {
            const totalIssues = results.reduce((sum, value) => sum + value, 0);
            this.issueEmitter.emit('total-issues', totalIssues);
        });
    }
    waitForResult(onDataPointChunksProcessed, isCollectResultMode) {
        this.out = [];
        this.issueEmitter.on('issues-sources', (issuesSources) => {
            if (!lodash_1.isEmpty(issuesSources)) {
                const issuesToOut = lodash_1.isArray(issuesSources) ? issuesSources : [issuesSources];
                issuesToOut.forEach(issue => {
                    if (isCollectResultMode) {
                        this.out.push(issue.view());
                    }
                    this.issueEmitter.emit('issue', issue.view());
                });
            }
            this.processedIssues++;
            this.done(onDataPointChunksProcessed);
        });
        this.issueEmitter.on('total-issues', (totalIssues) => {
            this.totalIssues = totalIssues;
            this.done(onDataPointChunksProcessed);
        });
        return this;
    }
    done(onDataPointChunksProcessed) {
        if (this.processedIssues === this.totalIssues) {
            onDataPointChunksProcessed(this.out);
        }
    }
}
exports.DataPointChunksProcessingStory = DataPointChunksProcessingStory;
