"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const async_1 = require("async");
const ddf_rules_1 = require("../ddf-rules");
const file_1 = require("../utils/file");
const shared_1 = require("../shared");
exports.isRecordRule = ruleObject => !!ruleObject.recordRule;
exports.isAggregativeRule = ruleObject => !!ruleObject.aggregateRecord && !!ruleObject.aggregativeRule;
class ProcessOneDataPointsChunkStory {
    constructor(fileDescriptors, issueEmitter) {
        this.fileDescriptors = fileDescriptors;
        this.issueEmitter = issueEmitter;
        this.sent = 0;
    }
    withCustomRules(customRules) {
        if (!lodash_1.isEmpty(customRules)) {
            this.customRules = customRules;
        }
        return this;
    }
    collect(ddfDataSet, issuesFilter, onDataCollected) {
        this.rulesWrappers = this.getRuleWrappers(issuesFilter);
        let isAbandoned = false;
        const actions = this.fileDescriptors.map((fileDescriptor) => (onFileDescriptorProcessed) => {
            if (isAbandoned) {
                return;
            }
            if (!isAbandoned && shared_1.supervisor.abandon) {
                isAbandoned = true;
                return;
            }
            file_1.walkFile(fileDescriptor.fullPath, (record, line) => {
                if (isAbandoned) {
                    return;
                }
                if (!isAbandoned && shared_1.supervisor.abandon) {
                    isAbandoned = true;
                    return;
                }
                for (let ruleWrapper of this.rulesWrappers) {
                    if (shared_1.sameTranslation(ruleWrapper.ruleKey, fileDescriptor) || shared_1.noTranslation(ruleWrapper.ruleKey, fileDescriptor)) {
                        if (exports.isRecordRule(ruleWrapper.rule)) {
                            const issues = ruleWrapper.rule.recordRule({ ddfDataSet, fileDescriptor, record, line });
                            this.issueEmitter.emit('issues-sources', issues);
                            this.sent++;
                        }
                        if (exports.isAggregativeRule(ruleWrapper.rule)) {
                            ruleWrapper.rule.aggregateRecord({ ddfDataSet, fileDescriptor, record, line }, ruleWrapper.storage);
                        }
                    }
                }
            }, onFileDescriptorProcessed);
        });
        async_1.parallelLimit(actions, shared_1.CONCURRENT_OPERATIONS_AMOUNT, onDataCollected);
    }
    analyze() {
        for (let ruleWrapper of this.rulesWrappers) {
            if (shared_1.supervisor.abandon) {
                return;
            }
            if (exports.isAggregativeRule(ruleWrapper.rule)) {
                const issues = ruleWrapper.rule.aggregativeRule(ruleWrapper.storage);
                if (!lodash_1.isEmpty(issues)) {
                    this.issueEmitter.emit('issues-sources', issues);
                    this.sent++;
                }
                ruleWrapper.storage = null;
            }
        }
    }
    getTotal() {
        return this.sent;
    }
    getRuleWrappers(issuesFilter) {
        let result = null;
        if (!this.customRules) {
            result = Object.getOwnPropertySymbols(ddf_rules_1.allRules)
                .filter(ruleKey => (exports.isAggregativeRule(ddf_rules_1.allRules[ruleKey]) || exports.isRecordRule(ddf_rules_1.allRules[ruleKey])) && issuesFilter.isAllowed(ruleKey))
                .map(ruleKey => ({
                ruleKey,
                rule: ddf_rules_1.allRules[ruleKey],
                storage: ddf_rules_1.allRules[ruleKey].getStorageTemplate ? ddf_rules_1.allRules[ruleKey].getStorageTemplate() : {}
            }));
        }
        if (this.customRules) {
            result = this.customRules.map(customRule => ({
                ruleKey: customRule.ruleKey,
                rule: customRule.rule,
                storage: customRule.rule.getStorageTemplate ? customRule.rule.getStorageTemplate() : {}
            }));
        }
        return result;
    }
}
exports.ProcessOneDataPointsChunkStory = ProcessOneDataPointsChunkStory;
