"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const events_1 = require("events");
const issues_filter_1 = require("./utils/issues-filter");
const ddf_data_set_1 = require("./ddf-definitions/ddf-data-set");
const shared_1 = require("./shared");
const data_point_chunks_processing_1 = require("./stories/data-point-chunks-processing");
const isExistInFilesChunks = (filesChunks, fileDescriptors) => {
    for (let chunk of filesChunks) {
        for (let fileDescriptor of fileDescriptors) {
            if (lodash_1.includes(chunk, fileDescriptor.fullPath)) {
                return true;
            }
        }
    }
    return false;
};
process.on('message', (message) => {
    const ddfDataSet = new ddf_data_set_1.DdfDataSet(message.rootPath, message.settings);
    const issuesFilter = new issues_filter_1.IssuesFilter(message.settings);
    const threadContext = { issuesFilter, ddfDataSet };
    const issueEmitter = new events_1.EventEmitter();
    const out = [];
    if (lodash_1.isEmpty(message.filesChunks) || message === 'abandon') {
        process.send({ err: null, out: [], finish: true });
        process.exit();
    }
    threadContext.ddfDataSet.load(() => {
        const fileDescriptorsChunks = shared_1.getAllDataPointFileDescriptorsChunks(ddfDataSet);
        const expectedFileDescriptorsChunks = [];
        for (let fileDescriptors of fileDescriptorsChunks) {
            if (isExistInFilesChunks(message.filesChunks, fileDescriptors)) {
                expectedFileDescriptorsChunks.push(fileDescriptors);
            }
        }
        const dataPointChunksProcessingStory = new data_point_chunks_processing_1.DataPointChunksProcessingStory(expectedFileDescriptorsChunks, issueEmitter);
        const theEnd = () => {
            process.send({ out, finish: true });
            process.exit();
        };
        issueEmitter.on('chunk-progress', () => {
            process.send({ progress: true });
        });
        issueEmitter.on('issue', (issue) => {
            if (issue) {
                if (!message.isCollectResultMode) {
                    process.send({ issue });
                }
                if (message.isCollectResultMode) {
                    out.push(issue);
                }
            }
        });
        dataPointChunksProcessingStory.waitForResult(theEnd, message.isCollectResultMode).processDataPointChunks(ddfDataSet, issuesFilter);
    });
});
