"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../ddf-rules/registry");
exports.WRONG_TAGS_COMBINATION_ERROR = new Error('Impossible combination of parameters: "include-tags" and "exclude-tags"');
exports.WRONG_RULES_COMBINATION_ERROR = new Error('Impossible combination of parameters: "include-rules" and "exclude-rules"');
class IssuesFilter {
    constructor(settings = {}) {
        this.includeTags = [];
        this.excludeTags = [];
        this.includeRules = [];
        this.excludeRules = [];
        if (settings.includeTags) {
            this.includeTags = settings.includeTags.split(' ');
        }
        if (settings.excludeTags) {
            this.excludeTags = settings.excludeTags.split(' ');
        }
        if (settings.includeRules) {
            this.includeRules = settings.includeRules.split(' ');
        }
        if (settings.excludeRules) {
            this.excludeRules = settings.excludeRules.split(' ');
        }
        this.checkIntegrity();
    }
    checkIntegrity() {
        if (!lodash_1.isEmpty(this.includeTags) && !lodash_1.isEmpty(this.excludeTags)) {
            throw exports.WRONG_TAGS_COMBINATION_ERROR;
        }
        if (!lodash_1.isEmpty(this.includeRules) && !lodash_1.isEmpty(this.excludeRules)) {
            throw exports.WRONG_RULES_COMBINATION_ERROR;
        }
    }
    isAllowedByTags(issueType) {
        if (!lodash_1.isEmpty(this.includeTags)) {
            return lodash_1.intersection(this.includeTags, registry_1.tags[issueType].map(tag => Symbol.keyFor(tag))).length > 0;
        }
        if (!lodash_1.isEmpty(this.excludeTags)) {
            return lodash_1.intersection(this.excludeTags, registry_1.tags[issueType].map(tag => Symbol.keyFor(tag))).length === 0;
        }
        return true;
    }
    isAllowedByRules(issueType) {
        if (!lodash_1.isEmpty(this.includeRules)) {
            return lodash_1.includes(this.includeRules, Symbol.keyFor(issueType));
        }
        if (!lodash_1.isEmpty(this.excludeRules)) {
            return !lodash_1.includes(this.excludeRules, Symbol.keyFor(issueType));
        }
        return true;
    }
    isAllowed(issue) {
        return this.isAllowedByTags(issue) && this.isAllowedByRules(issue);
    }
}
exports.IssuesFilter = IssuesFilter;
