"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const lodash_1 = require("lodash");
const winston_1 = require("winston");
const terminal_kit_1 = require("terminal-kit");
const args_1 = require("../utils/args");
const settings = args_1.getSettings();
const args = process.argv.join(' ');
const isSeparateThread = lodash_1.includes(args, 'thread.js');
class Progress {
    constructor(title, totalSteps) {
        this.title = title;
        this.isEnabled = +totalSteps > 1 && !isSeparateThread;
        if (this.isEnabled) {
            this.step = (100 / (+totalSteps)) / 100;
            this.progressValue = 0;
            this.progressBar = terminal_kit_1.terminal.progressBar({
                title: title,
                percent: true,
                inline: false,
                syncMode: true
            });
        }
        if (!this.isEnabled && this.progressBar) {
            this.resume();
        }
    }
    update() {
        if (this.isEnabled) {
            this.progressValue += this.step;
            this.progressBar.update(this.progressValue);
        }
    }
    resume() {
        if (this.progressBar) {
            this.progressBar.stop();
        }
    }
}
exports.Progress = Progress;
class ValidationTransport extends winston_1.Transport {
    constructor() {
        super();
        this.name = 'InfoTransport';
        this.level = 'notice';
        if (!settings.silent && !process.env.SILENT_MODE) {
            const dateLabel = new Date().toISOString().replace(/:/g, '');
            this.file = `validation-${dateLabel}.log`;
        }
    }
    updateSettings(newSettings) {
        const newSettingsKeys = Object.keys(newSettings);
        for (let key of newSettingsKeys) {
            settings[key] = newSettings[key];
        }
    }
    log(level, msg, meta, callback) {
        if (level === 'progressInit' && !settings.silent && !process.env.SILENT_MODE) {
            if (this.progress) {
                this.progress.resume();
            }
            this.progress = new Progress(msg, meta.total);
        }
        if (level === 'progress' && !settings.silent && !process.env.SILENT_MODE) {
            this.progress.update();
        }
        if (level === 'notice') {
            if (settings.silent) {
                console.log(msg);
            }
            if (!settings.silent && !process.env.SILENT_MODE) {
                fs.appendFile(this.file, msg, (err) => {
                    if (err) {
                        return callback(err);
                    }
                    callback(null, true);
                });
            }
        }
    }
}
exports.ValidationTransport = ValidationTransport;
exports.validationTransport = new ValidationTransport();
let logger;
exports.getLogger = () => {
    if (!logger) {
        logger = new winston_1.Logger({
            levels: {
                trace: 0,
                progressInit: 1,
                progress: 2,
                notice: 3,
                warning: 4,
                error: 5
            },
            level: 'notice',
            transports: [
                exports.validationTransport
            ]
        });
    }
    return logger;
};
exports.getTransport = () => exports.validationTransport;
